
# LinguaPlone support
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

from AccessControl import ClassSecurityInfo
from Products.CMFCore import permissions
from xml.sax import make_parser, handler, SAXException
from xml.sax.saxutils import *
from xbelplone import ImportHandler
import random, string
from DateTime import DateTime
from config import VIEW_CONTENTS_PERMISSION, ADD_CONTENT_PERMISSION, LINK_TYPES


""" 
   ATBookmarks allows you to manage your Bookmarks online in a Folder
"""

schema = BaseSchema +  Schema((
        StringField('id',
                 required=1, 
                 mode="rw",
                 accessor="getId",
                 mutator="setId",
                 default=None,
                 widget=IdWidget(
                 label="Short Name",
                 label_msgid="label_short_name",
                 description="Should not contain spaces, underscores or mixed case. "\
                 "Short Name is part of the item's web address.",
                 description_msgid="help_shortname",
                 visible={'view' : 'visible'},
                 i18n_domain="plone"),
                 ),
    StringField('title',
                required=1,
                searchable=1,
                default='',
                accessor='Title',
                widget=StringWidget(label="Title",
                                    label_msgid="label_title",
                                    description_msgid="help_title",
                                    i18n_domain="plone"),
                ),    
    StringField('description',
                isMetadata=1,
                accessor='Description',
                widget=TextAreaWidget(label='Description', 
                                label_msgid="label_Description",
                                description='A brief description of this item.',
                                description_msgid="help_description",
                                i18n_domain="plone"), 
               ),
    ))
    
new_actions = (
    {'id': 'bookmarklivefeed',
     'name': 'bookmark RSS Feed',
     'action': "string:${object_url}/bookmark-rss.xml",
     'permissions': (permissions.View,),
     'category': 'document_actions',
     },
     {'id': 'xbelxml',
     'name': 'xbel XML',
     'action': "string:${object_url}/xbel.xml",
     'permissions': (permissions.View,),
     'category': 'document_actions',
     },
     {'id': 'tastysibebar',
     'name': 'Add Tasty Sidebar',
     'action': "string:javascript: addNetscapePanel('${object_url}/tasty_sidebar');",
     'permissions': (permissions.View,),
     'category': 'document_actions',
     },
     )
     
def modify_fti(fti):
    fti['actions'] = fti['actions'] + new_actions

        
class BookmarkFolder(OrderedBaseFolder):
    """
        A Bookmark folder lets you manage your Bookmarks online
    """

    schema = schema
    filter_content_types = 1
    allowed_content_types = ('SpotlightLink', 'Link', 'Favorite', 'Image', 'BookmarkFolder') 
    meta_type = 'BookmarkFolder'
    archetype_name = 'Bookmark Folder'
    _at_rename_after_creation = True    
    content_icon = 'bookmark_folder.png'
    immediate_view = 'base_edit'
    default_view   = 'bookmark_view'
    suppl_views    = ('tasty_view', 'tasty_tree')
    global_allow = True

    actions = ({
       'id': 'view',
        'name': 'View',
        'action': 'string:${object_url}/',
        'permissions': (permissions.View,)
        },
       {'id'            : 'upload',
        'name'          : 'Upload',
        'action'        : 'string:${object_url}/upload_xbel_form',
        'permissions'   : (permissions.ModifyPortalContent, ),
        'visible':0
         },
        {'id': 'references',
          'name': 'References',
          'action': 'string:${object_url}/reference_edit',
          'permissions': (permissions.ModifyPortalContent,),
          'visible':0
          },
        {'id': 'local_roles',
          'name': 'Sharing',
          'action': 'string:${object_url}/sharing',
          'permissions': (permissions.ManageProperties),
          'visible':1
          },
        {'id': 'metadata',
          'name': 'Properties',
          'action': 'string:${object_url}/properties',
          'permissions': (permissions.ModifyPortalContent,),
          'visible':1
          })    
    aliases = {
        '(Default)'  : 'bookmark_view',
        'view'       : 'bookmark_view',
        'index.html' : '',
        'edit'       : 'base_edit',
        'properties' : 'base_metadata',
        'sharing'    : 'folder_localrole_form',
        'gethtml'    : '',
        'mkdir'      : '',
        }
          
    # Standard security settings
    security = ClassSecurityInfo()

    security.declareObjectProtected(VIEW_CONTENTS_PERMISSION)

    security.declareProtected(ADD_CONTENT_PERMISSION, 'mkBookmarkId')
    def mkBookmarkId(self, anchor=None):
	""" generate a sane BookmarkId out of the URL """
	import string, random
	id = ''
	try:
    	    splitAnchor = string.split(anchor,'//')
	    tmpId=splitAnchor [1] [:80]
	    for x in tmpId:
        	if (('A' <= x <='Z') or ('a' <= x <= 'z') or ('0' <= x <='9')):
           	    id = id + x
            	else:
           	    id = id + '-'
	except:
   	   id = 'lnk_' + str(int( context.ZopeTime()))
           id = id + str(random.randint(100,999))
	return id

    security.declareProtected(ADD_CONTENT_PERMISSION, 'bookmarkAdd')
    def bookmarkAdd(self, remote_url, id=None, title=None, description=None, keywords=None):
	""" Add a Bookmark """
    	request = self.REQUEST
    	RESPONSE =  request.RESPONSE
	if id == None:
	   id = mkBookmarkId(self,remote_url)
	if title == None:
	   title = remote_url
	self.invokeFactory( 'Link', id=id, title=title, remote_url=remote_url, description=description)
	view_url = '%s/%s' % (self.absolute_url(),
                      self.getTypeInfo().getActionById('view')
                     )
	new_link = getattr(self, id)
	try:
    	    new_keywords = tuple(keywords.split(' '))
	except:
	    new_keywords = None
	try:
	   new_link.setSubject(new_keywords)
	   new_link.reindexObject()
	except:
	   pass

	msg = 'portal_status_message=\'%s\' has been added as a Bookmark' % title
	return RESPONSE.redirect('%s?%s' % (view_url, msg))

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'excludeSubject')
    def excludeSubject(self, this_url=None, subject=None):
	"""Exclude a subjects from filter"""
	subjectList = this_url.split('/')
	new_subjects=[]
	for lsubject in subjectList:
    	    if lsubject != subject:
       		new_subjects.append(lsubject)  
        new_subjects.sort()     	
	return '/'.join(new_subjects)
	
    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'includeSubject')
    def includeSubject(self, this_url=None, subject=None):
	"""Include this subjects into filter"""	
	subjectList = this_url.split('/')		
	subjectList.append(subject)			 	
	subjectList.sort()
	return '/'.join(subjectList)
        
    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getLinksBySubject')
    def getLinksBySubject(self, subject=[None,], sortorder=None, sorton=None):
	"""=Filter the Links and Favorites by subject
	 Subject should be a tuple of keywords or None.
 	The 'Subject' operator is 'AND'!"""
	lfolderpath = '/'.join(self.getPhysicalPath())
	try:
	  subject.remove(None)
	except:
	  pass	
	try:
	   subject = tuple(subject)
	except:
	   subject=(None,)

	if subject == (None,) or subject == ('',) or subject ==():
	   links = self.portal_catalog.searchResults(portal_type=LINK_TYPES,
                                     sort_on=sorton,
                                     sort_order=sortorder,
                                     path=lfolderpath)
	else:
	   links =  self.portal_catalog.searchResults(portal_type=LINK_TYPES,
                                     sort_on=sorton,
                                     sort_order=sortorder,
                                     path=lfolderpath,
                                     Subject={'query' : subject,
                                              'operator' : 'and'})
	return links


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getLinkSubjects')
    def getLinkSubjects(self):
	"""=get the subjects for Links and Favorites in this context"""        
	lfolderpath = '/'.join(self.getPhysicalPath())
	links = self.portal_catalog.searchResults(portal_type=LINK_TYPES, path=lfolderpath)
	# Use dictionary keys to get unique values
	subjects = {}
	for link in links:
	     subjs = link.Subject
	     for subj in subjs:
	         subjects[subj] = None
	subjs = subjects.keys()
	subjs.sort()
	return subjs
    
    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getRefinementSubjects')
    def getRefinementSubjects(self, Links =None, Keywords=None):
        """ get the subjects and their counts that are a refinement for this query """
        subjects ={}
        if Links and Keywords:   	   
   	   for kw in Keywords:
       		subjects[kw] = 0
   	   for link in Links:
       		subjs = link.Subject
       		for kw in subjs:
	   	    subjects[kw] = subjects[kw] +1
	return subjects
    
    security.declareProtected(ADD_CONTENT_PERMISSION, 'getMoreBmfKeywords')
    def getMoreBmfKeywords(self, Keywords=[]):
        """ Get the all subjects except those passed in the Keyword argument"""
        allKeywords = self.portal_catalog.uniqueValuesFor('Subject')
        try: 
           otherKw = []
           for kw in allKeywords:
              if kw not in Keywords:
                 otherKw.append(kw)
           return otherKw
        except:
           return []
           
    security.declareProtected(ADD_CONTENT_PERMISSION, 'getFolderKeywords')
    def getFolderKeywords(self):
        """ get all Keywords for the current Bookmarkfolder as default for new links"""
        Keywords =()
        try:
           Keywords = self.Subject()
           return Keywords
        except:
           return []
        
    def fromFile(self, stream):
        """ Import bookmarks from a file into this folder """
        m = ImportHandler()
        parser = make_parser()
        parser.setContentHandler(m)
        parser.parse(stream)
        f = m.currentFolder
        self.convertFolder(self, f)
        
    def convertFolder(self, ploneFolder, xbelFolder):
        """ Add links and BookmarkFolders to current folder """
        xbelFolder.bookmarks.sort()
        #xbelFolder.bookmarks.reverse()
        description = ""
        for b in xbelFolder.bookmarks:
                try:
                    id = b.id.encode('ascii','ignore')
                except:
                    id =  mkBookmarkId(self, anchor=b.url)
                title=b.name.encode('utf-8','ignore')
                url=b.url.encode('ascii','ignore')
                description=b.desc.encode('utf-8','ignore')
                try: 
                    ploneFolder.invokeFactory('Link', id=id, title=title, remote_url=url, description=description)
                except:
                    pass
                
        for f in xbelFolder.subfolders:
            try: 
            	id=f.id.encode('ascii','ignore')
            except:
                id= 'bmf_' + str(random.randint(1000,99999))
            try:
                ploneFolder.invokeFactory('BookmarkFolder', id=id, title=f.name.encode('utf-8','ignore'), description=f.desc.encode('utf-8','ignore'))            
            except:
                pass
            try:
                folder = getattr(ploneFolder, f.id)
                self.convertFolder(folder, f)
            except:
                pass
         
registerType(BookmarkFolder)
